      REAL FUNCTION alog1p(xs)
C     (alog1p)
C     Return alog(1 + xs), taking care to avoid subtraction loss.
C
C     This version does internal computations in double precision.
C     (17-Jun-2002)
      REAL xs
      DOUBLE PRECISION x
      DOUBLE PRECISION CUTLO, CUTHI, ONE, ZERO
      DOUBLE PRECISION sum, term, xn, xtons
C
C     CUTLO = -1/beta, for arbitrary base beta
C
      DATA CUTLO / -0.5d+00 /
C
C     CUTHI = 1/beta, for arbitrary base beta
C
      DATA CUTHI / +0.5d+00 /
C
      DATA ONE   / 1.0d+00 /
      DATA ZERO  / 0.0d+00 /
C
      x = dble(xs)
C
C     We handle the computation in three regions:
C
C     x in [-Infinity, CUTLO):  log(1+x)
C     x in [CUTLO, CUTHI]:      Taylor series
C     x in (CUTHI, Infinity]:   log(1+x)
C
C     The central region suffers loss of one or more bits if the
C     simple formula is used.
C
C     We also handle the cases of alog1p(NaN) and alog1p(0) specially,
C     so as to preserve NaNs, and the sign of zero.
C
      IF (x .ne. x) THEN
          alog1p = xs
      ELSE IF (x .eq. ZERO) THEN
          alog1p = xs
      ELSE IF (x .lt. CUTLO) THEN
          alog1p = sngl(dlog(ONE + x))
      ELSE IF (x .le. CUTHI) THEN
          term = x
          sum = ZERO
          xn = ONE
          xtons = x
   10     IF (sum + term .ne. sum) THEN
              sum = sum + term
              xn = xn + ONE
              xtons = -xtons * x
              term = xtons / xn
              GO TO 10
          END IF
          alog1p = sngl(sum)
      ELSE
          alog1p = sngl(dlog(ONE + x))
      END IF
C
      END

      REAL FUNCTION al1p10(x)
C     (log1p10)
C     Return alog10(1 + x), taking care to avoid subtraction loss.
C     (17-Jun-2002)
      REAL x
      REAL alog1p
      REAL LOG10E
C
      DATA LOG10E
     X    /0.43429448190325182765112891891660508229439700580366E+00/
C
      al1p10 = alog1p(x) * LOG10E
C
      END
