      DOUBLE PRECISION FUNCTION dexpm1(x)
C     (dexpm1)
C     Return (exp(x) - 1), taking care to avoid subtraction loss.
C
C     This version uses a [8,2]-degree rational minimax polynomial
C     computed in Maple by
C
C         with(numapprox):
C         Digits := 30:
C         minimax((exp(x)-1)/x, x = ln(1/2)..ln(3/2), [8,2], 1, 'err');
C         printf("%.2e\n", err);
C
C     The reported absolute error is 4.46e-17, below the IEEE 754
C     machine epsilon of 2**(-52) = 2.22e-16.  This is the lowest
C     degree, and thus optimal, minimax polynomial whose error is below
C     that limit.
C     (28-Jun-2002)
      DOUBLE PRECISION x
      DOUBLE PRECISION CUTLO, CUTHI, ONE, ZERO
C
C     CUTLO = ln(0.5) (really, ln(1-1/beta) for arbitrary base beta)
C
      DATA CUTLO/-0.6931471805599453094172321214581765680755001343603D0/
C
C     CUTHI = ln(1.5) (really, ln(1+1/beta) for arbitrary base beta)
C
      DATA CUTHI/+0.4054651081081643819780131154643491365719904234625D0/
C
      DATA ZERO / 0.0D+00 /
      DATA ONE  / 1.0D+00 /
C
C     We handle the computation in three regions:
C
C     x in [-Infinity, CUTLO):  exp(x) - 1
C     x in [CUTLO, CUTHI]:      minimax polynomial
C     x in (CUTHI, Infinity]:   exp(x) - 1
C
C     The central region suffers loss of one or more bits if the
C     simple formula is used.
C
C     The following IF statements handle the case of NaN, signed zero,
C     and the three regions above.
C
      IF (x .ne. x) THEN
          dexpm1 = x
      ELSE IF (x .eq. ZERO) THEN
          dexpm1 = x
      ELSE IF (x .lt. CUTLO) THEN
          dexpm1 = dexp(x) - ONE
      ELSE IF (x .le. CUTHI) THEN
          dexpm1 = x*((0.972760965309405311622155720413d+00 +
     X        (0.307787828517447727384884772352d+00 +
     X        (0.818657798196538507995540410710d-01 +
     X        (0.152839042000987783573773325934d-01 +
     X        (0.217086067815840319776222026328d-02 +
     X        (0.239254749790356045755155302865d-03 +
     X        (0.202567665043690345972014473134d-04 +
     X        (0.123997064618335651098991731845d-05 +
     X        0.440230411775549299639177088088d-07 *
     X        x) * x) * x) * x) * x) * x) * x) * x) /
     X        (0.972760965309405292862421978316d+00 +
     X        (-0.178592654137254098688418858833d+00 +
     X        0.903527933671750503621315618349d-02 * x) * x))
      ELSE
          dexpm1 = dexp(x) - ONE
      ENDIF
C
      END
