      DOUBLE PRECISION FUNCTION dlog1p(x)
C     (log1p)
C     Return dlog(1 + x), taking care to avoid subtraction loss.
C
C     This version uses an alternate form of the Taylor series that
C     requires only about a quarter as many terms, and has only positive
C     terms.
C     (17-Jun-2002)
      DOUBLE PRECISION x
      DOUBLE PRECISION CUTLO, CUTHI, ONE, TWO, ZERO
      DOUBLE PRECISION sum, term, twonp1, z, zsq, zsqton
C
C     CUTLO = -1/beta, for arbitrary base beta
C
      DATA CUTLO / -0.5d+00 /
C
C     CUTHI = 1/beta, for arbitrary base beta
C
      DATA CUTHI / +0.5d+00 /
C
      DATA ONE   / 1.0d+00 /
      DATA TWO   / 2.0d+00 /
      DATA ZERO  / 0.0d+00 /
C
C     We handle the computation in three regions:
C
C     x in [-Infinity, CUTLO):  log(1+x)
C     x in [CUTLO, CUTHI]:      alternate Taylor series
C     x in (CUTHI, Infinity]:   log(1+x)
C
C     The central region suffers loss of one or more bits if the
C     simple formula is used.
C
C     We also handle the cases of log1p(NaN) and log1p(0) specially,
C     so as to preserve NaNs, and the sign of zero.
C
      IF (x .ne. x) THEN
          dlog1p = x
      ELSE IF (x .eq. ZERO) THEN
          dlog1p = x
      ELSE IF (x .lt. CUTLO) THEN
          dlog1p = dlog(ONE + x)
      ELSE IF (x .le. CUTHI) THEN
          sum = ZERO
          term = ONE
          twonp1 = ONE
          z = x / (TWO + x)
          zsq = z * z
          zsqton = ONE
   10     IF ((sum + term) .ne. sum) THEN
              sum = sum + term
              twonp1 = twonp1 + TWO
              zsqton = zsqton * zsq
              term = zsqton / twonp1
              GO TO 10
          END IF
          dlog1p = z * (sum + sum)
      ELSE
          dlog1p = dlog(ONE + x)
      END IF
C
      END

      DOUBLE PRECISION FUNCTION dl1p10(x)
C     (log1p10)
C     Return dlog10(1 + x), taking care to avoid subtraction loss.
C     (17-Jun-2002)
      DOUBLE PRECISION x
      DOUBLE PRECISION dlog1p
      DOUBLE PRECISION LOG10E
C
      DATA LOG10E
     X    /0.43429448190325182765112891891660508229439700580366D+00/
C
      dl1p10 = dlog1p(x) * LOG10E
C
      END
