      REAL FUNCTION expm1(x)
C     (expm1)
C     Return (exp(x) - 1), taking care to avoid subtraction loss.
C
C     This version uses a [2,3]-degree rational minimax polynomial
C     computed in Maple by
C
C         with(numapprox):
C         Digits := 20:
C         minimax((exp(x)-1)/x, x = ln(1/2)..ln(3/2), [2,3], 1, 'err');
C         printf("%.2e\n", err);
C
C     The reported absolute error is 7.38e-09, below the IEEE 754
C     machine epsilon of 2**(-23) = 1.19e-07.  This is the lowest
C     degree, and thus optimal, minimax polynomial whose error is below
C     that limit.
C     (28-Jun-2002)
      REAL x
      REAL CUTLO, CUTHI, ONE, ZERO
C
C     CUTLO = ln(0.5) (really, ln(1-1/beta) for arbitrary base beta)
C
      DATA CUTLO/-0.6931471805599453094172321214581765680755001343603E0/
C
C     CUTHI = ln(1.5) (really, ln(1+1/beta) for arbitrary base beta)
C
      DATA CUTHI/+0.4054651081081643819780131154643491365719904234625E0/
C
      DATA ZERO / 0.0E+00 /
      DATA ONE  / 1.0E+00 /
C
C     We handle the computation in three regions:
C
C     x in [-Infinity, CUTLO):  exp(x) - 1
C     x in [CUTLO, CUTHI]:      minimax polynomial
C     x in (CUTHI, Infinity]:   exp(x) - 1
C
C     The central region suffers loss of one or more bits if the
C     simple formula is used.
C
C     The following IF statements handle the case of NaN, signed zero,
C     and the three regions above.
C
      IF (x .ne. x) THEN
          expm1 = x
      ELSE IF (x .eq. ZERO) THEN
          expm1 = x
      ELSE IF (x .lt. CUTLO) THEN
          expm1 = exp(x) - ONE
      ELSE IF (x .le. CUTHI) THEN
          expm1 = x * (
     X                 (0.91672752602583573933 +
     X                  (-0.46711394671763984969e-2 +
     X                   0.15252661708262483600e-1 * x) * x) /
     X                 (0.91672752934412321018 +
     X                  (-0.46303497404987656531 +
     X                   (0.93982016782958864518e-1 -
     X                    0.80140563231405006715e-2 * x) * x) * x))
      ELSE
          expm1 = exp(x) - ONE
      ENDIF
C
      END
