C     (Time operations)
C     Run a timing test for qexpm1() and qexp(), and report the results
C     on stdout.
C     (12-Jun-2002)
      integer MAXRAN
      integer MAXTIM
C
C     For 16KB on-chip data cache:
C
C     parameter (MAXRAN = 800)
C     parameter (MAXTIM = 4 * 100)
C
C     For 4MB secondary cache
C
C     parameter (MAXRAN = 60 * 1024)
C     parameter (MAXTIM = 5)
C
C     Force into memory:
C
      parameter (MAXRAN = 625 * 512)
      parameter (MAXTIM = 1)
C
      double precision nanos
      double precision t1, t2
C
      real*16 CUTLO, CUTHI
      real*16 qexpm1, qran
      real*16 r(MAXRAN)
C
      integer j, k
C
C     CUTLO = ln(0.5) (really, ln(1-1/beta) for arbitrary base beta)
C
      DATA CUTLO/-0.6931471805599453094172321214581765680755001343603Q0/
C
C     CUTHI = ln(1.5) (really, ln(1+1/beta) for arbitrary base beta)
C
      DATA CUTHI/+0.4054651081081643819780131154643491365719904234625Q0/
C
C     Make a table of pseudo-random numbers in the range
C     (CUTHI,CUTLO), so that our later calls to qexpm1() exercise only
C     the Taylor series computation.  The table size is chosen to fit
C     in a 1MB cache, leaving some room for other data.
C
      do 10 k = 1,MAXRAN
           r(k) = CUTLO + qran(k)*(CUTHI - CUTLO)
   10 continue
      do 15 k = 1,MAXRAN
           call store(r(k))
   15 continue
C
C     Run the timing loop for qexpm1(), with a dummy interior call to
C     prevent optimizers from eliminating the loop body.
C
      t1 = nanos()
      do 30 j = 1, MAXTIM
          do 20 k = 1, MAXRAN
              call store(qexpm1(r(k)))
   20     continue
   30 continue
      t2 = nanos()
      write (6,'('' Average qexpm1() time: '', F12.0, '' nsec'')')
     X    (t2 - t1)/(dfloat(MAXTIM) * dfloat(MAXRAN))
C
C     Run the timing loop for qexp(), with a dummy interior call to
C     prevent optimizers from eliminating the loop body.
C
      t1 = nanos()
      do 50 j = 1, MAXTIM
          do 40 k = 1, MAXRAN
              call store(qexp(r(k)))
   40     continue
   50 continue
      t2 = nanos()
      write (6,'('' Average qexp() time:   '', F12.0, '' nsec'')')
     X    (t2 - t1)/(dfloat(MAXTIM) * dfloat(MAXRAN))
      end

      subroutine store(x)
      real*16 x
      end
