      CALL TDEXPM1(6)
      END
      SUBROUTINE  TDEXPM1 (IOUT)
      EXTERNAL     DEXPM1, DRAN
C
C     PROGRAM TO TEST DEXPM1
C
C     DATA REQUIRED
C
C        NONE
C
C     SUBPROGRAMS REQUIRED FROM THIS PACKAGE
C
C        MACHAR - AN ENVIRONMENTAL INQUIRY PROGRAM PROVIDING
C                 INFORMATION ON THE FLOATING-POINT ARITHMETIC
C                 SYSTEM.  NOTE THAT THE CALL TO MACHAR CAN
C                 BE DELETED PROVIDED THE FOLLOWING FOUR
C                 PARAMETERS ARE ASSIGNED THE VALUES INDICATED
C
C                 IBETA - THE RADIX OF THE FLOATING-POINT SYSTEM
C                 IT    - THE NUMBER OF BASE-IBETA DIGITS IN THE
C                         SIGNIFICAND OF A FLOATING-POINT NUMBER
C                 XMIN  - THE SMALLEST NON-VANISHING FLOATING-POINT
C                         POWER OF THE RADIX
C                 XMAX  - THE LARGEST FINITE FLOATING-POINT NO.
C
C        DRAN(K) - A FUNCTION SUBPROGRAM RETURNING RANDOM REAL
C                 NUMBERS UNIFORMLY DISTRIBUTED OVER (0,1)
C
C
C     STANDARD FORTRAN SUBPROGRAMS REQUIRED
C
C         DABS, DINT, DLOG, DMAX1, DEXPM1, DFLOAT, DSQRT
C
C     LATEST REVISION - JUNE 10, 2002
C
C     AUTHOR - NELSON H. F. BEEBE
C              MATHEMATICS DEPARTMENT, UNIVERSITY OF UTAH
C
C     BASED ON TEXP:
C
C     LATEST REVISION - DECEMBER 6, 1979
C
C     AUTHOR - W. J. CODY
C              ARGONNE NATIONAL LABORATORY
C
C
      INTEGER I,IBETA,IEXP,IOUT,IRND,IT,I1,J,JP1,K1,K2,K3,MACHEP,
     X        MAXEXP,MINEXP,N,NEGEP,NGRD
      DOUBLE PRECISION A,AIT,ALBETA,B,BETA,C16,C45,DEL,EPS,EPSNEG,
     X                 FIVE,HALF,ONE,ONEP5,R6,R7,TWO,V,W,X,XL,XMAX,
     X                 XMIN,XN,X1,Y,Z,ZERO,ZZ,ZZZ
      DOUBLE PRECISION DEXPM1, DRAN, DSTORE
C
C     IOUT = 6
C
      CALL MACHAR(IBETA,IT,IRND,NGRD,MACHEP,NEGEP,IEXP,MINEXP,
     X            MAXEXP,EPS,EPSNEG,XMIN,XMAX)
      BETA = DFLOAT(IBETA)
      ALBETA = DLOG(BETA)
      AIT = DFLOAT(IT)
C
      ZERO = 0.0D+00
      HALF = 0.5D+00
      ONE = 1.0D+00
      ONEP5 = 1.5D+00
      TWO = 2.0D+00
      FIVE = 5.0D+00
      C16 = 16.0D+00
      C45 = 45.0D+00
C
      N = 2000
      XN = DFLOAT(N)
      I1 = 0
C-----------------------------------------------------------------------
C     RANDOM ARGUMENT ACCURACY TESTS
C-----------------------------------------------------------------------
      DO 300 JP1 = 1, 6
         J = JP1 - 1
         K1 = 0
         K3 = 0
         X1 = ZERO
         R6 = ZERO
         R7 = ZERO
C
         IF (J .NE. 0) GO TO 81
         A = -DLOG(0.9D+00 * XMAX)
         B = DLOG(HALF*EPSNEG)
         V = ONE / C16
         GO TO 86
C
   81    IF (J .NE. 1) GO TO 82
         A = DLOG(HALF*EPSNEG)
         B = -FIVE * DLOG(TWO)
         V = ONE / C16
         GO TO 86
C
   82    IF (J .NE. 2) GO TO 83
         A = -FIVE * DLOG(TWO)
         B = DLOG(HALF)
         V = C45 / C16
         GO TO 86
C
   83    IF (J .NE. 3) GO TO 84
         A = DLOG(HALF)
         B = -XMIN
         V = ONE / C16
         GO TO 86
C
   84    IF (J .NE. 4) GO TO 85
         A = XMIN
         B = DLOG(ONEP5)
         V = C45 / C16
         GO TO 86
C
   85    IF (J .NE. 5) GO TO 86
         A = DLOG(ONEP5)
         B = DLOG(0.9D+00 * XMAX)
         V = ONE / C16
C
   86    DEL = (B - A) / XN
         XL = A
C
         DO 200 I = 1, N
            X = XL + DEL * DRAN(I1)
C-----------------------------------------------------------------------
C     PURIFY ARGUMENTS
C-----------------------------------------------------------------------
            Y = DSTORE(X - V)
            X = DSTORE(Y + V)
C-----------------------------------------------------------------------
C           WE HAVE AN ADDITIVE FORMULA, SUITABLE FOR USE WITH
C           UNIFORMLY-DISTRIBUTED PSEUDO-RANDOM ARGUMENTS:
C
C           DEXPM1(X - V) = DEXPM1(-V)*(1 + DEXPM1(X)) + DEXPM1(X)
C                        = DEXPM1(X)*(1 + DEXPM1(-V)) + DEXPM1(-V)
C
C           AND WE REPRESENT DEXPM1(V) AS C1 + C2, WHERE C1 IS EXACTLY
C           REPRESENTABLE, AND C2 IS A SMALL CORRECTION TERM.
C-----------------------------------------------------------------------
            Z = DEXPM1(X)
            ZZ = DEXPM1(Y)
            IF (J .NE. 0) GO TO 101
C
C           J .EQ. 0, V .EQ. 1/16, EXPM1(-V) = -1/16 + 0.00191306...
C
            ZZZ = (Z * 0.9375D+00) + (Z *
     X      1.9130628134757861197108246223050845246808905494418220D-03)
     X      + (-0.0625D+00 +
     X      1.9130628134757861197108246223050845246808905494418220D-03)
            GO TO 110
  101       IF (J .NE. 1) GO TO 102
C
C           J .EQ. 1, V .EQ. 1/16, DEXPM1(-V) = -1/16 + 0.00191306...
C
            ZZZ = (Z * 0.9375D+00) + (Z *
     X      1.9130628134757861197108246223050845246808905494418220D-03)
     X      + (-0.0625D+00 +
     X      1.9130628134757861197108246223050845246808905494418220D-03)
            GO TO 110
  102       IF (J .NE. 2) GO TO 103
C
C           J .EQ. 2, V .EQ. 45/16, DEXPM1(-V) = -(15/16 + 0.0024453...)
C
            ZZZ = (Z * 0.0625D+00) - (Z *
     X      2.4453321046920570389443866922096486559947441807771923D-03)
     X      - (0.9375D+00 +
     X      2.4453321046920570389443866922096486559947441807771923D-03)
            GO TO 110
  103       IF (J .NE. 3) GO TO 104
C
C           J .EQ. 3, V .EQ. 1/16, DEXPM1(-V) = -1/16 + 0.00191306...
C
            ZZZ = (Z * 0.9375D+00) + ((Z *
     X      1.9130628134757861197108246223050845246808905494418220D-03)
     X      + (-0.0625D+00 +
     X      1.9130628134757861197108246223050845246808905494418220D-03))
            GO TO 110
  104       IF (J .NE. 4) GO TO 105
C
C           J .EQ. 4, V .EQ. 45/16, EXPM1(-V) = -(15/16 + 0.0024453...)
C
            ZZZ = (Z * 0.0625D+00) - (Z *
     X      2.4453321046920570389443866922096486559947441807771923D-03)
     X      - (0.9375D+00 +
     X      2.4453321046920570389443866922096486559947441807771923D-03)
            GO TO 110
C
C           J .EQ. 5, V .EQ. 1/16, EXPM1(-V) = -1/16 + 0.00191306...
C
  105       IF (J .NE. 5) GO TO 110
            ZZZ = (Z * 0.9375D+00) + ((Z *
     X      1.9130628134757861197108246223050845246808905494418220D-03)
     X      + (-0.0625D+00 +
     X      1.9130628134757861197108246223050845246808905494418220D-03))
            GO TO 110
C
  110       W = ONE
            IF (ZZZ .NE. ZERO) W = (ZZ - ZZZ) / ZZZ
            IF (W .LT. ZERO) K1 = K1 + 1
            IF (W .GT. ZERO) K3 = K3 + 1
            W = DABS(W)
            IF (W .LE. R6) GO TO 120
            R6 = W
            X1 = X
  120       R7 = R7 + W*W
C           XL = XL + DEL
            XL = A + DFLOAT(I)*DEL
  200    CONTINUE
C
         K2 = N - K3 - K1
         R7 = DSQRT(R7/XN)
         WRITE (IOUT,1000) V, -V
         WRITE (IOUT,1010) N,A,B
         WRITE (IOUT,1011) K1,K2,K3
         WRITE (IOUT,1020) IT,IBETA
         W = -999.0D+00
         IF (R6 .NE. ZERO) W = DLOG(DABS(R6))/ALBETA
         WRITE (IOUT,1021) R6,IBETA,W,X1
         W = DMAX1(AIT+W,ZERO)
         WRITE (IOUT,1022) IBETA,W
         W = -999.0D+00
         IF (R7 .NE. ZERO) W = DLOG(DABS(R7))/ALBETA
         WRITE (IOUT,1023) R7,IBETA,W
         W = DMAX1(AIT+W,ZERO)
         WRITE (IOUT,1022) IBETA,W
  300 CONTINUE
C-----------------------------------------------------------------------
C     SPECIAL TESTS
C-----------------------------------------------------------------------
      WRITE (IOUT,1025)
      WRITE (IOUT,1030)
C
      DO 320 I = 1, 5
         X = DRAN(I1) * BETA
         Z = ONE + DEXPM1(X) + DEXPM1(X)/DEXPM1(-X)
         WRITE (IOUT,1060) X, Z
  320 CONTINUE
C
      WRITE (IOUT,1040)
      X = ZERO
      Y = DEXPM1(X) - ONE
      WRITE (IOUT,1041) Y
      X = DINT(DLOG(XMIN))
      Y = DEXPM1(X)
      WRITE (IOUT,1042) X, Y
      X = DINT(DLOG(XMAX))
      Y = DEXPM1(X)
      WRITE (IOUT,1042) X, Y
      X = X / TWO
      V = X / TWO
      Y = DEXPM1(X)
      Z = DEXPM1(V)
      Z = Z * Z
      WRITE (IOUT,1043) X, Y, V, Z
C-----------------------------------------------------------------------
C     TESTS OF IEEE 754 SUBNORMALS (IF AVAILABLE AND ENABLED)
C-----------------------------------------------------------------------
      IF (XMIN * HALF .EQ. ZERO) GO TO 400
      WRITE (IOUT, 1070)
      K1 = 0
      K2 = 0
      W = EPS
      Z = XMIN
C
C     THE CHOICE OF X HERE ENSURES THAT ALL SIGNIFICANT BITS ARE ONE, SO
C     THAT WE CAN DETECT DEFICIENT IMPLEMENTATIONS OF DEXPM1() THAT LOSE
C     LOW-ORDER BITS.
C
  330 IF (Z .EQ. ZERO) GO TO 340
      X = Z * (ONE - W)
      Y = DEXPM1(X)
      IF (Y .NE. X) K1 = K1 + 1
      IF (Y .NE. X) WRITE (IOUT, 1071) X, Y, X
      Y = DEXPM1(-X)
      IF (Y .NE. (-X)) K1 = K1 + 1
      IF (Y .NE. (-X)) WRITE (IOUT, 1071) -X, Y, -X
      K2 = K2 + 2
      Z = Z * HALF
      W = W + W
      GO TO 330
  340 IF (K1 .EQ. 0) WRITE (IOUT, 1072) K2
      IF (K1 .NE. 0) WRITE (IOUT, 1073) K1, K2
C-----------------------------------------------------------------------
C     TEST OF ERROR RETURNS
C-----------------------------------------------------------------------
  400 WRITE (IOUT,1050)
      X = -ONE / DSQRT(XMIN)
      WRITE (IOUT,1052) X
      Y = DEXPM1(X)
      WRITE (IOUT,1061) Y
      X = -X
      WRITE (IOUT,1052) X
      Y = DEXPM1(X)
      WRITE (IOUT,1061) Y
      WRITE (IOUT,1100)
      RETURN
C-----------------------------------------------------------------------
 1000 FORMAT(18H1TEST OF DEXPM1(X-, F7.4, 12H) VS DEXPM1(, F7.4,
     X     29H)*(1 + DEXPM1(X)) + DEXPM1(X)//)
 1010 FORMAT(I7,47H RANDOM ARGUMENTS WERE TESTED FROM THE INTERVAL /
     X 6X,1H(,E15.4E4,1H,,E15.4E4,1H)//)
 1011 FORMAT(23H DEXPM1(X+V) WAS LARGER,I6,7H TIMES, /
     X     13X,7H AGREED,I6,11H TIMES, AND /
     X   9X,11HWAS SMALLER,I6,7H TIMES.//)
 1020 FORMAT(10H THERE ARE,I4,5H BASE,I4,
     X    46H SIGNIFICANT DIGITS IN A FLOATING-POINT NUMBER  //)
 1021 FORMAT(30H THE MAXIMUM RELATIVE ERROR OF,E15.4E4,3H = ,I4,3H **,
     X  F7.2/4X,16HOCCURRED FOR X =,E17.6E4)
 1022 FORMAT(27H THE ESTIMATED LOSS OF BASE,I4,
     X  22H SIGNIFICANT DIGITS IS,F7.2//)
 1023 FORMAT(40H THE ROOT MEAN SQUARE RELATIVE ERROR WAS,E15.4E4,
     X    3H = ,I4,3H **,F7.2)
 1025 FORMAT(14H1SPECIAL TESTS//)
 1030 FORMAT(52H THE IDENTITY  1 + DEXPM1(X) = -DEXPM1(X)/DEXPM1(-X),
     1       17H  WILL BE TESTED.
     2      //8X,1HX,9X,21H1 + F(X) + F(X)/F(-X)/)
 1040 FORMAT(//26H TEST OF SPECIAL ARGUMENTS //)
 1041 FORMAT(21H DEXPM1(0.0) - 1.0 = ,E15.6E4/)
 1042 FORMAT(8H DEXPM1(,E15.6E4,4H) = ,E15.6E4/)
 1043 FORMAT(/11H IF DEXPM1(,E15.6E4,4H) = ,E15.6E4,13H IS NOT ABOUT /
     X 8H DEXPM1(,E15.6E4,7H)**2 = ,E15.6E4,
     X 26H THERE IS AN ARG RED ERROR)
 1050 FORMAT(22H1TEST OF ERROR RETURNS  //)
 1052 FORMAT(/40H DEXPM1 WILL BE CALLED WITH THE ARGUMENT,E15.4E4/
     X       37H THIS SHOULD TRIGGER AN ERROR MESSAGE//)
 1060 FORMAT(2E16.6E4/)
 1061 FORMAT(27H DEXPM1 RETURNED THE VALUE ,E15.4E4///)
 1070 FORMAT(//37H TEST OF IEEE 754 SUBNORMAL ARGUMENTS/)
 1071 FORMAT(15H ERROR: DEXPM1(, E15.6E4, 11H) RETURNED ,E15.6E4,
     X    20H INSTEAD OF CORRECT ,E15.6E4)
 1072 FORMAT(5H ALL , I3,
     1       50H SUBNORMALS TESTED CORRECTLY SATISFY DEXPM1(X) = X//)
 1073 FORMAT(I3, 4H OF , I3,
     1       48H SUBNORMALS TESTED FAIL TO SATISFY DEXPM1(X) = X//)
 1100 FORMAT(25H THIS CONCLUDES THE TESTS )
C     ---------- LAST CARD OF DEXPM1 TEST PROGRAM ----------
      END
