      CALL TQLOG1P(6)
      END
      SUBROUTINE  TQLOG1P (IOUT)
      EXTERNAL     QRAN
C     PROGRAM TO TEST QLOG1P/QL1P10
C
C     DATA REQUIRED
C
C        NONE
C
C     SUBPROGRAMS REQUIRED FROM THIS PACKAGE
C
C        MACHAR - AN ENVIRONMENTAL INQUIRY PROGRAM PROVIDING
C                 INFORMATION ON THE FLOATING-POINT ARITHMETIC
C                 SYSTEM.  NOTE THAT THE CALL TO MACHAR CAN
C                 BE DELETED PROVIDED THE FOLLOWING FOUR
C                 PARAMETERS ARE ASSIGNED THE VALUES INDICATED
C
C                 IBETA - THE RADIX OF THE FLOATING-POINT SYSTEM
C                 IT    - THE NUMBER OF BASE-IBETA DIGITS IN THE
C                         SIGNIFICAND OF A FLOATING-POINT NUMBER
C                 XMIN  - THE SMALLEST NON-VANISHING FLOATING-POINT
C                         POWER OF THE RADIX
C                 XMAX  - THE LARGEST FINITE FLOATING-POINT NO.
C
C        QRAN(K) - A FUNCTION SUBPROGRAM RETURNING RANDOM REAL
C                 NUMBERS UNIFORMLY DISTRIBUTED OVER (0,1)
C
C
C     STANDARD FORTRAN SUBPROGRAMS REQUIRED
C
C         QABS, QEXP, QLOG1P, QL1P10, QMAX1, QFLOAT, QSIGN, QSQRT
C
C
C     LATEST REVISION - JUNE 19, 2002
C
C     AUTHOR - NELSON H. F. BEEBE
C              MATHEMATICS DEPARTMENT, UNIVERSITY OF UTAH
C
C     BASED ON TQLOG:
C
C     LATEST REVISION - DECEMBER 6, 1979
C
C     AUTHOR - W. J. CODY
C              ARGONNE NATIONAL LABORATORY
C
C
      INTEGER I,IBETA,IEXP,IOUT,IRND,IT,I1,J,K1,K2,K3,MACHEP,
     X        MAXEXP,MINEXP,N,NEGEP,NGRD
      REAL*16 A,AIT,ALBETA,B,BETA,C,D56,D78,DEL,EIGHT,EPS,
     X     EPSNEG,HALF,ONE,R6,R7,S,TENTH,W,X,XL,XMAX,XMIN,XN,
     X     X1,Y,Z,ZERO,ZZ
      REAL*16 QLOG1P, QL1P10, QRAN, QSTORE
C
C     IOUT = 6
C
      CALL MACHAR(IBETA,IT,IRND,NGRD,MACHEP,NEGEP,IEXP,MINEXP,
     X            MAXEXP,EPS,EPSNEG,XMIN,XMAX)
      BETA = QFLOAT(IBETA)
      ALBETA = QLOG(BETA)
      AIT = QFLOAT(IT)
      J = IT / 3
      ZERO = 0.0Q+00
      HALF = 0.5Q+00
      EIGHT = 8.0Q+00
      TENTH = 0.1Q+00
      ONE = 1.0Q+00
C
C     FOR REGIONS 5 AND 6, WHERE THE SCALE FACTOR IN THE IDENTITY TEST
C     IS 17/16, FIND D56 = (SMALLEST 1/BETA**K SUCH THAT THE RELATIVE
C     ERROR IN SUBTRACTING LN(S) FROM LN(S*(1 + 1/BETA**K)) IS LARGER
C     THAN 1/BETA), GUARANTEEING NO LEADING BIT LOSS.
C
      S = 17.0Q+00 / 16.0Q+00
      D56 = (ONE + EPS) * (QEXP(BETA * QLOG(S) / (BETA - ONE)))/S - ONE
C
C     DO THE SAME FOR REGIONS 7 AND 8.
C
      S = 11.0Q+00 / 10.0Q+00
      D78 = (ONE + EPS) * (QEXP(BETA * QLOG(S) / (BETA - ONE)))/S - ONE
C
      C = ONE
C
      DO 50 I = 1, J
         C = C / BETA
   50 CONTINUE
C
      A = -C
      B = +C
      N = 2000
      XN = QFLOAT(N)
      I1 = 0
C-----------------------------------------------------------------------
C     RANDOM ARGUMENT ACCURACY TESTS
C-----------------------------------------------------------------------
      DO 300 J = 1, 8
         K1 = 0
         K3 = 0
         X1 = ZERO
         R6 = ZERO
         R7 = ZERO
         DEL = (B - A) / XN
         XL = A
C
         DO 200 I = 1, N
            X = DEL * QRAN(I1) + XL
C
            IF (J .NE. 1) GO TO 100
            Y = X
            ZZ = QLOG1P(X)
            Z = ONE / 3.0Q+00
            Z = Y * (Z - Y / 4.0Q+00)
            Z = (Z - HALF) * Y * Y + Y
            GO TO 150
C
  100       IF (J .NE. 2) GO TO 110
            X = QSTORE(QSTORE(X + EIGHT) - EIGHT)
            Y = X + X / 16.0Q+00
            Z = QLOG1P(X - ONE)
C#          ZZ = QLOG1P(Y - ONE) - (LOG(17/16) = 31/512 + DELTA)
C
C           ORIGINAL SUMMATION ORDER:
C
C           ZZ = QLOG1P(Y - ONE) -
C    X      7.774681643484258060613204042026328620247514472377081452Q-05
C           ZZ = ZZ - 31.0Q+00 / 512.0Q+00
C
C           REVISED SUMMATION ORDER (SMALLEST TO LARGEST):
C
            ZZ = QLOG1P(Y - ONE) - (31.0Q+00 / 512.0Q+00 +
     X      7.77468164348425806061320404202632862024751447237708145Q-05)
            GO TO 150
C
  110       IF (J .NE. 3) GO TO 120
            X = QSTORE(QSTORE(X + EIGHT) - EIGHT)
            Y = X + X * TENTH
            Z = QL1P10(X - ONE)
C#          ZZ = QL1P10(Y - ONE) - (LOG10(1.1) = 21/512 + DELTA)
            ZZ = QL1P10(Y - ONE) -
     X      3.770601582250407501999712430242417067021904664530945965Q-04
            ZZ = ZZ - 21.0Q+00 / 512.0Q+00
            GO TO 150
C
  120       IF (J .NE. 4) GO TO 130
            Z = QLOG1P(X*X - ONE)
            ZZ = QLOG1P(X - ONE)
            ZZ = ZZ + ZZ
            GO TO 150
C
  130       IF ((J .NE. 5) .AND. (J .NE. 6)) GO TO 140
C
C           WITH X IN (0.5,1.5), ADDING AND SUBTRACTING 16 CLEARS 4
C           LOW-ORDER BITS, SO THAT 17X/16 IS EXACTLY COMPUTABLE.
C
            X = QSTORE(QSTORE(X + 16.0Q+00) - 16.0Q+00)
            Y = X + X / 16.0Q+00
            Z = QLOG1P(X - ONE)
C#          ZZ = QLOG1P(Y - ONE) - (LOG(17/16) = 31/512 + DELTA)
C
C           ORIGINAL SUMMATION ORDER:
C
C           ZZ = QLOG1P(Y - ONE) -
C    X      7.774681643484258060613204042026328620247514472377081452Q-05
C           ZZ = ZZ - 31.0Q+00 / 512.0Q+00
C
C           REVISED SUMMATION ORDER (SMALLEST TO LARGEST):
C
            ZZ = QLOG1P(Y - ONE) - (31.0Q+00 / 512.0Q+00 +
     X      7.77468164348425806061320404202632862024751447237708145Q-05)
            GO TO 150
C
  140       IF ((J .NE. 7) .AND. (J .NE. 8)) GO TO 150
            X = QSTORE(QSTORE(X + 16.0Q+00) - 16.0Q+00)
            Y = X + X * TENTH
            Z = QL1P10(X - ONE)
C#          ZZ = QL1P10(Y - ONE) - (LOG10(11/10) = 21/512 + DELTA)
            ZZ = QL1P10(Y - ONE) -
     X      3.770601582250407501999712430242417067021904664530945965Q-04
            ZZ = ZZ - 21.0Q+00 / 512.0Q+00
            GO TO 150
C
  150       W = ONE
            IF (Z .NE. ZERO) W = (Z - ZZ) / Z
            Z = QSIGN(W,Z)
            IF (Z .GT. ZERO) K1 = K1 + 1
            IF (Z .LT. ZERO) K3 = K3 + 1
            W = QABS(W)
            IF (W .LE. R6) GO TO 160
            R6 = W
            X1 = X
  160       R7 = R7 + W*W
C           XL = XL + DEL
            XL = A + QFLOAT(I)*DEL
  200    CONTINUE
C
         K2 = N - K3 - K1
         R7 = QSQRT(R7/XN)
         IF (J .EQ. 1) WRITE (IOUT,1000)
         IF (J .EQ. 2) WRITE (IOUT,1001) IBETA
         IF (J .EQ. 3) WRITE (IOUT,1005) IBETA
         IF (J .EQ. 4) WRITE (IOUT,1002)
         IF (J .EQ. 5) WRITE (IOUT,1001) IBETA
         IF (J .EQ. 6) WRITE (IOUT,1001) IBETA
         IF (J .EQ. 7) WRITE (IOUT,1005) IBETA
         IF (J .EQ. 8) WRITE (IOUT,1005) IBETA
         IF (J .EQ. 1) WRITE (IOUT,1009) N,C
         IF (J .NE. 1) WRITE (IOUT,1010) N,A,B
         IF ((J .NE. 3) .AND. (J .NE. 7) .AND. (J .NE. 8))
     X      WRITE (IOUT,1011) K1,K2,K3
         IF ((J .EQ. 3) .OR.  (J .EQ. 7) .OR. (J .EQ. 8))
     X      WRITE (IOUT,1012) K1,K2,K3
         WRITE (IOUT,1020) IT,IBETA
         W = -999.0Q+00
         IF (R6 .NE. ZERO) W = QLOG(QABS(R6))/ALBETA
         WRITE (IOUT,1021) R6,IBETA,W,X1
         W = QMAX1(AIT+W,ZERO)
         WRITE (IOUT,1022) IBETA,W
         W = -999.0Q+00
         IF (R7 .NE. ZERO) W = QLOG(QABS(R7))/ALBETA
         WRITE (IOUT,1023) R7,IBETA,W
         W = QMAX1(AIT+W,ZERO)
         WRITE (IOUT,1022) IBETA,W
C
         IF (J .NE. 1) GO TO 230
         A = QSQRT(HALF)
         B = 15.0Q+00 / 16.0Q+00
         GO TO 300
C
  230    IF (J .NE. 2) GO TO 240
         A = QSQRT(TENTH)
         B = 0.9Q+00
         GO TO 300
C
  240    IF (J .NE. 3) GO TO 250
         A = 16.0Q+00
         B = 240.0Q+00
C
  250    IF (J .NE. 4) GO TO 260
         A = ONE - ONE/BETA
         B = ONE - D56
         GO TO 300
C
  260    IF (J .NE. 5) GO TO 270
         A = ONE + D56
         B = ONE + ONE/BETA
         GO TO 300
C
  270    IF (J .NE. 6) GO TO 280
         A = ONE - ONE/BETA
         B = ONE - D78
         GO TO 300
C
  280    IF (J .NE. 7) GO TO 300
         A = ONE + D78
         B = ONE + ONE/BETA
         GO TO 300
C
  300 CONTINUE
C-----------------------------------------------------------------------
C     SPECIAL TESTS
C-----------------------------------------------------------------------
      WRITE (IOUT,1025)
      WRITE (IOUT,1030)
C
      DO 320 I = 1, 5
         X = QRAN(I1)
         X = X + X + 15.0Q+00
         Y = ONE / X
         Z = QLOG1P(X - ONE) + QLOG1P(Y - ONE)
         ZZ = Z / EPS
         WRITE (IOUT,1060) X, Z, ZZ
  320 CONTINUE
C
      WRITE (IOUT,1035)
C
      X = QSQRT(EPS + EPS + EPS) / BETA
      DO 330 I = 1, 5
         Y = QLOG1P(X)
         Z = Y - (X - HALF*X*X)
         ZZ = Z / EPS
         WRITE (IOUT,1065) X, Z, ZZ
         X = -X
         Y = QLOG1P(X)
         Z = Y - (X - HALF*X*X)
         ZZ = Z / EPS
         WRITE (IOUT,1065) X, Z, ZZ
         X = -X * HALF
  330 CONTINUE
C
      WRITE (IOUT,1040)
      X = -ZERO
      Y = QLOG1P(X)
      WRITE (IOUT,1041) X, Y
      X = ZERO
      Y = QLOG1P(X)
      WRITE (IOUT,1041) X, Y
      X = -(ONE - EPSNEG)
      Y = QLOG1P(X)
      WRITE (IOUT,1041) X, Y
      X = -EPS
      Y = QLOG1P(X)
      WRITE (IOUT,1041) X, Y
      X = EPS
      Y = QLOG1P(X)
      WRITE (IOUT,1041) X, Y
      X = -(EPS + EPS/(BETA*BETA*BETA*BETA))
      Y = QLOG1P(X)
      WRITE (IOUT,1041) X, Y
      X = EPS + EPS/(BETA*BETA*BETA*BETA)
      Y = QLOG1P(X)
      WRITE (IOUT,1041) X, Y
      X = XMIN
      Y = QLOG1P(X)
      WRITE (IOUT,1042) X, Y
      X = XMAX
      Y = QLOG1P(X)
      WRITE (IOUT,1043) X, Y
C-----------------------------------------------------------------------
C     TESTS OF IEEE 754 SUBNORMALS (IF AVAILABLE AND ENABLED)
C-----------------------------------------------------------------------
      IF (XMIN * HALF .EQ. ZERO) GO TO 400
      WRITE (IOUT, 1070)
      K1 = 0
      K2 = 0
      W = EPS
      Z = XMIN
C
C     THE CHOICE OF X HERE ENSURES THAT ALL SIGNIFICANT BITS ARE ONE, SO
C     THAT WE CAN DETECT DEFICIENT IMPLEMENTATIONS OF QLOG1P() THAT LOSE
C     LOW-ORDER BITS.
C
  340 IF (Z .EQ. ZERO) GO TO 350
      X = Z * (ONE - W)
      Y = QLOG1P(X)
      IF (Y .NE. X) K1 = K1 + 1
      IF (Y .NE. X) WRITE (IOUT, 1071) X, Y, X
      Y = QLOG1P(-X)
      IF (Y .NE. (-X)) K1 = K1 + 1
      IF (Y .NE. (-X)) WRITE (IOUT, 1071) -X, Y, -X
      K2 = K2 + 2
      Z = Z * HALF
      W = W + W
      GO TO 340
  350 IF (K1 .EQ. 0) WRITE (IOUT, 1072) K2
      IF (K1 .NE. 0) WRITE (IOUT, 1073) K1, K2
C-----------------------------------------------------------------------
C     TEST OF ERROR RETURNS
C-----------------------------------------------------------------------
  400 WRITE (IOUT,1050)
      X = -2.0Q+00
      WRITE (IOUT,1052) X
      Y = QLOG1P(X)
      WRITE (IOUT,1055) Y
      X = -ONE
      WRITE (IOUT,1052) X
      Y = QLOG1P(X)
      WRITE (IOUT,1055) Y
      WRITE (IOUT,1100)
      RETURN
C-----------------------------------------------------------------------
 1000 FORMAT(49H1TEST OF QLOG1P(X) VS T.S. EXPANSION OF QLOG1P(X)  //)
 1001 FORMAT(23H1TEST OF QLOG1P(X-1) VS,
     X    33H QLOG1P(17(X-1)/16)-QLOG1P(17/16)//
     X    12H IF BASE (= , I2, 22H) IS NOT A POWER OF 2,,
     X    50H EXPECT BIT LOSSES 1 OR 2 BITS LARGER THAN NORMAL.//)
 1002 FORMAT(47H1TEST OF QLOG1P((X-1)*(X-1)) VS 2 * QLOG1P(X-1)  //)
 1005 FORMAT(23H1TEST OF QL1P10(X-1) VS,
     X    33H QL1P10(11(X-1)/10)-QL1P10(11/10) //
     X    12H IF BASE (= , I2, 12H) IS NOT 10,,
     X    50H EXPECT BIT LOSSES 1 OR 2 BITS LARGER THAN NORMAL.//)
 1009 FORMAT(I7,47H RANDOM ARGUMENTS WERE TESTED FROM THE INTERVAL /
     X 6X,24H(-EPS,+EPS), WHERE EPS =, E15.4E4//)
 1010 FORMAT(I7,47H RANDOM ARGUMENTS WERE TESTED FROM THE INTERVAL /
     X 6X,1H(,E15.4E4,1H,,E15.4E4,1H)//)
 1011 FORMAT(21H QLOG1P(X) WAS LARGER,I6,7H TIMES, /
     X     14X,7H AGREED,I6,11H TIMES, AND /
     X     10X,11HWAS SMALLER,I6,7H TIMES.//)
 1012 FORMAT(21H QL1P10(X) WAS LARGER,I6,7H TIMES, /
     X     14X,7H AGREED,I6,11H TIMES, AND /
     X     10X,11HWAS SMALLER,I6,7H TIMES.//)
 1020 FORMAT(10H THERE ARE,I4,5H BASE,I4,
     X    46H SIGNIFICANT DIGITS IN A FLOATING-POINT NUMBER  //)
 1021 FORMAT(30H THE MAXIMUM RELATIVE ERROR OF,E15.4E4,3H = ,I4,3H **,
     X  F7.2/4X,16HOCCURRED FOR X =,E17.6E4)
 1022 FORMAT(27H THE ESTIMATED LOSS OF BASE,I4,
     X  22H SIGNIFICANT DIGITS IS,F7.2//)
 1023 FORMAT(40H THE ROOT MEAN SQUARE RELATIVE ERROR WAS,E15.4E4,
     X    3H = ,I4,3H **,F7.2)
 1025 FORMAT(14H1SPECIAL TESTS//)
 1030 FORMAT(45H THE IDENTITY  QLOG1P(X-1) = -QLOG1P((1/X)-1),
     X      17H  WILL BE TESTED.
     X      //8X,1HX,9X,19HF(X-1) + F((1/X)-1)/)
 1035 FORMAT(/37H THE IDENTITY  QLOG1P(X) = X - X**2/2,
     X      50H  FOR ABS(X) .LE. SQRT(3*EPS)/BETA WILL BE TESTED./
     X      48H NONZERO RESULTS SUGGEST THAT QLOG1P(X) HAS BEEN,
     X      34H SLOPPILY IMPLEMENTED AS QLOG(1+X)
     X      //8X,1HX,9X,20HF(X) - F(X - X**2/2)/)
 1040 FORMAT(//26H TEST OF SPECIAL ARGUMENTS //)
 1041 FORMAT(8H QLOG1P(, E16.7E4, 4H) = ,E16.7E4//)
 1042 FORMAT(23H QLOG1P(XMIN) = QLOG1P(,E16.7E4,4H) = ,E16.7E4//)
 1043 FORMAT(23H QLOG1P(XMAX) = QLOG1P(,E16.7E4,4H) = ,E16.7E4//)
 1050 FORMAT(22H1TEST OF ERROR RETURNS//)
 1052 FORMAT(40H QLOG1P WILL BE CALLED WITH THE ARGUMENT,E15.4E4/
     X       37H THIS SHOULD TRIGGER AN ERROR MESSAGE//)
 1055 FORMAT(27H QLOG1P RETURNED THE VALUE ,E15.4E4///)
 1060 FORMAT(F15.7,E16.7E4,F15.2,5H ULPS/)
 1065 FORMAT(2E16.7E4,F15.2,5H ULPS/)
 1070 FORMAT(37H TEST OF IEEE 754 SUBNORMAL ARGUMENTS/)
 1071 FORMAT(15H ERROR: QLOG1P(, E15.6E4, 11H) RETURNED ,E15.6E4,
     X    20H INSTEAD OF CORRECT ,E15.6E4)
 1072 FORMAT(5H ALL , I3,
     X       50H SUBNORMALS TESTED CORRECTLY SATISFY QLOG1P(X) = X//)
 1073 FORMAT(I3, 4H OF , I3,
     X       48H SUBNORMALS TESTED FAIL TO SATISFY QLOG1P(X) = X//)
 1100 FORMAT(25H THIS CONCLUDES THE TESTS )
C     ---------- LAST CARD OF QLOG1P/QL1P10 TEST PROGRAM ----------
      END
